@@ -18,7 +18,7 @@
  *
  * The author may be contacted via http://dmalloc.com/
  *
- * $Id: chunk.c,v 1.201 2004/01/28 14:29:43 gray Exp $
+ * $Id: chunk.c,v 1.207 2004/10/19 14:50:43 gray Exp $
  */
 
 /*
@@ -151,7 +151,6 @@ static	unsigned long	alloc_tot_pnts = 0;	/* total pointers */
 static	unsigned long	heap_check_c = 0;	/* count of heap-checks */
 static	unsigned long	user_block_c = 0;	/* count of blocks */
 static	unsigned long	admin_block_c = 0;	/* count of admin blocks */
-static	unsigned long	extern_block_c = 0;	/* count of extern blocks */
 
 /* alloc counts */
 static	unsigned long	func_malloc_c = 0;	/* count the mallocs */
@@ -454,16 +453,8 @@ static	int	insert_slot(skip_alloc_t *slot_p, const int free_b)
  *
  * level_n -> Number of the level we are looking for.  Set to 0 to
  * have it be chosen at random.
- *
- * extern_mem_pp <- Pointer to a void * which will be set to a chunk
- * of memory that was allocated externally and needs to be accounted
- * for.
- *
- * extern_np <- Pointer to an integer which will be set to the number
- * of blocks in the external memory chunk.
  */
-static	void	*alloc_slots(const int level_n, void **extern_mem_pp,
-			     int *extern_np)
+static	void	*alloc_slots(const int level_n)
 {
   skip_alloc_t	*new_p;
   entry_block_t	*block_p;
@@ -475,7 +466,7 @@ static	void	*alloc_slots(const int level_n, void **extern_mem_pp,
   }
   
   /* we need to allocate a new block of the slots of this level */
-  block_p = _dmalloc_heap_alloc(BLOCK_SIZE, extern_mem_pp, extern_np);
+  block_p = _dmalloc_heap_alloc(BLOCK_SIZE);
   if (block_p == NULL) {
     /* error code set in _dmalloc_heap_alloc */
     return NULL;
@@ -604,8 +595,8 @@ static	int	remove_slot(skip_alloc_t *delete_p, skip_alloc_t *update_p)
 static	skip_alloc_t	*get_slot(void)
 {
   skip_alloc_t	*new_p;
-  int		level_n, extern_n, slot_size;
-  void		*admin_mem, *extern_mem;
+  int		level_n, slot_size;
+  void		*admin_mem;
   
   /* generate the level for our new slot */
   level_n = random_level(MAX_SKIP_LEVEL);
@@ -628,19 +619,16 @@ static	skip_alloc_t	*get_slot(void)
    * So we are trying to get a slot of a certain size to store
    * something in a skip list.  We didn't have any on the free-list so
    * now we will allocate a block.  We allocate a block of memory to
-   * hold the slots and we might note that there was an external use
-   * of sbrk.  This means that we may need 2 of the new slots to
-   * account for the admin and external memory in addition to the 1
-   * requested.
+   * hold the slots meaning that we may need 1 new slot to account for
+   * the admin and external memory in addition to the 1 requested.
    *
    * To do it right, would take a recursive call to get_slot which I
-   * am not willing to do so we will have 2 blocks in a row (sometimes
-   * 3) which have the same height.  This is less than efficient but
-   * oh well.
+   * am not willing to do so we will have 2 blocks in a row which have
+   * the same height.  This is less than efficient but oh well.
    */
   
   /* add in all of the unused slots to the linked list */
-  admin_mem = alloc_slots(level_n, &extern_mem, &extern_n);
+  admin_mem = alloc_slots(level_n);
   if (admin_mem == NULL) {
     /* error code set in alloc_slots */
     return NULL;
@@ -670,37 +658,6 @@ static	skip_alloc_t	*get_slot(void)
     return NULL;
   }
   
-  /*
-   * if we got any external blocks when we did our allocation then
-   * account for them
-   */
-  if (extern_n > 0) {
-    /* get one for the admin memory */
-    new_p = entry_free_list[level_n];
-    if (new_p == NULL) {
-      /*
-       * HUH?  This isn't right.  We should have created a whole bunch
-       * of addresses
-       */
-      dmalloc_errno = ERROR_ADDRESS_LIST;
-      dmalloc_error("get_slot");
-      return NULL;
-    }
-    entry_free_list[level_n] = new_p->sa_next_p[0];
-    memset(new_p, 0, slot_size);
-    new_p->sa_flags = ALLOC_FLAG_EXTERN;
-    new_p->sa_mem = extern_mem;
-    new_p->sa_total_size = extern_n * BLOCK_SIZE;
-    new_p->sa_level_n = level_n;
-    extern_block_c++;
-    
-    /* now put it in the used list */
-    if (! insert_slot(new_p, 0 /* used list */)) {
-      /* error reported above */
-      return NULL;
-    }
-  }
-  
   /* now get one for the user */
   new_p = entry_free_list[level_n];
   if (new_p == NULL) {
@@ -772,49 +729,6 @@ static	skip_alloc_t	*insert_address(void *address, const int free_b,
   return new_p;
 }
 
-/*
- * static void *allocate_memory
- *
- * DESCRIPTION:
- *
- * Allocate memory on the heap and account for external blocks.
- *
- * RETURNS:
- *
- * Success - Valid slot pointer.
- *
- * Failure - NULL
- *
- * ARGUMENTS:
- *
- * size -> Size of the memory that we want to allocate.
- */
-static	void	*allocate_memory(const unsigned int size)
-{
-  skip_alloc_t	*slot_p;
-  void		*mem, *extern_mem;
-  int		extern_n;
-  
-  /* allocate the memory and record if there were any external blocks */
-  mem = _dmalloc_heap_alloc(size, &extern_mem, &extern_n);
-  if (mem == HEAP_ALLOC_ERROR) {
-    /* error code set in _dmalloc_heap_alloc */
-    return NULL;
-  }
-  
-  if (extern_n > 0) {
-    slot_p = insert_address(extern_mem, 0 /* used */, extern_n * BLOCK_SIZE);
-    if (slot_p == NULL) {
-      /* error code set in insert_address */
-      return NULL;
-    }
-    /* set the flags to be external memory */ 
-    slot_p->sa_flags = ALLOC_FLAG_EXTERN;
-  }
-  
-  return mem;
-}
-
 /******************************* misc routines *******************************/
 
 /*
@@ -932,6 +846,7 @@ static	void	get_pnt_info(const skip_alloc_t *slot_p, pnt_info_t *info_p)
 {
   info_p->pi_fence_b = BIT_IS_SET(slot_p->sa_flags, ALLOC_FLAG_FENCE);
   info_p->pi_valloc_b = BIT_IS_SET(slot_p->sa_flags, ALLOC_FLAG_VALLOC);
+  info_p->pi_blanked_b = BIT_IS_SET(slot_p->sa_flags, ALLOC_FLAG_BLANK);
   
   info_p->pi_alloc_start = slot_p->sa_mem;
   
@@ -1067,15 +982,10 @@ static	char	*display_pnt(const void *user_pnt, const skip_alloc_t *alloc_p,
  *
  * now_line -> Line number from where we generated the error.
  *
- * prev_file -> File of a previous location.  For instance the
- * location of a previous free() if we tried to free something twice.
+ * user_pnt -> Pointer in question.  This can be 0L then it will use
+ * the slot_p memory pointer.
  *
- * prev_line -> Line number of a previous location.  For instance the
- * location of a previous free() if we tried to free something twice.
- *
- * user_pnt -> Pointer in question.
- *
- * size -> Size that the user requested including any fence space.
+ * slot_p -> Pointer to the slot associated with the user_pnt or 0L.
  *
  * reason -> Reason string why something happened.
  *
@@ -1084,39 +994,58 @@ static	char	*display_pnt(const void *user_pnt, const skip_alloc_t *alloc_p,
  */
 static	void	log_error_info(const char *now_file,
 			       const unsigned int now_line,
-			       const char *prev_file,
-			       const unsigned int prev_line,
-			       const void *user_pnt, const unsigned int size,
+			       const void *user_pnt,
+			       const skip_alloc_t *slot_p,
 			       const char *reason, const char *where)
 {
   static int	dump_bottom_b = 0, dump_top_b = 0;
   char		out[(DUMP_SPACE + FENCE_BOTTOM_SIZE + FENCE_TOP_SIZE) * 4];
   char		where_buf[MAX_FILE_LENGTH + 64];
   char		where_buf2[MAX_FILE_LENGTH + 64];
-  const char	*reason_str;
+  const char	*prev_file;
   const void	*dump_pnt = user_pnt;
+  const void	*start_user;
+  unsigned int	prev_line, user_size;
+  skip_alloc_t	*other_p;
+  pnt_info_t	pnt_info;
   int		out_len, dump_size, offset;
   
-  /* get a proper reason string */
-  if (reason == NULL) {
-    reason_str = dmalloc_strerror(dmalloc_errno);
+  dmalloc_error(where);
+
+  if (slot_p == NULL) {
+    prev_file = NULL;
+    prev_line = 0;
+    user_size = 0;
+    start_user = user_pnt;
   }
   else {
-    reason_str = reason;
+    prev_file = slot_p->sa_file;
+    prev_line = slot_p->sa_line;
+    user_size = slot_p->sa_user_size;
+    if (user_pnt == NULL) {
+      get_pnt_info(slot_p, &pnt_info);
+      start_user = pnt_info.pi_user_start;
+    }
+    else {
+      start_user = user_pnt;
+    }
   }
   
+  /* get a proper reason string */
+  if (reason != NULL) {
+    dmalloc_message("  error details: %s", reason);
+  }
   /* dump the pointer information */
-  if (user_pnt == NULL) {
-    dmalloc_message("%s: %s: from '%s' prev access '%s'",
-		    where, reason_str,
+  if (start_user == NULL) {
+    dmalloc_message("  from '%s' prev access '%s'",
 		    _dmalloc_chunk_desc_pnt(where_buf, sizeof(where_buf),
 					    now_file, now_line),
 		    _dmalloc_chunk_desc_pnt(where_buf2, sizeof(where_buf2),
 					    prev_file, prev_line));
   }
   else {
-    dmalloc_message("%s: %s: pointer '%#lx' from '%s' prev access '%s'",
-		    where, reason_str, (unsigned long)user_pnt,
+    dmalloc_message("  pointer '%#lx' from '%s' prev access '%s'",
+		    (unsigned long)start_user,
 		    _dmalloc_chunk_desc_pnt(where_buf, sizeof(where_buf),
 					    now_file, now_line),
 		    _dmalloc_chunk_desc_pnt(where_buf2, sizeof(where_buf2),
@@ -1140,18 +1069,22 @@ static	void	log_error_info(const char *now_file,
     if (! dump_bottom_b) {
       out_len = expand_chars(fence_bottom, FENCE_BOTTOM_SIZE, out,
 			     sizeof(out));
-      dmalloc_message("Dump of proper fence-bottom bytes: '%.*s'",
+      dmalloc_message("  dump of proper fence-bottom bytes: '%.*s'",
 		      out_len, out);
       dump_bottom_b = 1;
     }
     offset = -FENCE_BOTTOM_SIZE;
     dump_size = DUMP_SPACE + FENCE_BOTTOM_SIZE;
+    if (dump_size > user_size + FENCE_OVERHEAD_SIZE) {
+      dump_size = user_size + FENCE_OVERHEAD_SIZE;
+    }
   }
-  else if (dmalloc_errno == ERROR_OVER_FENCE && size > 0) {
+  else if (dmalloc_errno == ERROR_OVER_FENCE
+	   && user_size > 0) {
     /* NOTE: only dump out the proper fence-post area once */
     if (! dump_top_b) {
       out_len = expand_chars(fence_top, FENCE_TOP_SIZE, out, sizeof(out));
-      dmalloc_message("Dump of proper fence-top bytes: '%.*s'",
+      dmalloc_message("  dump of proper fence-top bytes: '%.*s'",
 		      out_len, out);
       dump_top_b = 1;
     }
@@ -1159,30 +1092,60 @@ static	void	log_error_info(const char *now_file,
      * The size includes the bottom fence post area.  We want it to
      * align with the start of the top fence post area.
      */
-    offset = size - FENCE_BOTTOM_SIZE - FENCE_TOP_SIZE;
-    if (offset < 0) {
-      offset = 0;
+    if (DUMP_SPACE > user_size + FENCE_OVERHEAD_SIZE) {
+      dump_size = user_size + FENCE_OVERHEAD_SIZE;
+      offset = -FENCE_BOTTOM_SIZE;
+    }
+    else {
+      dump_size = DUMP_SPACE;
+      /* we will go backwards possibly up to FENCE_BOTTOM_SIZE offset */
+      offset = user_size + FENCE_TOP_SIZE - DUMP_SPACE;
     }
-    dump_size = DUMP_SPACE + FENCE_TOP_SIZE;
   }
   else {
     dump_size = DUMP_SPACE;
     offset = 0;
+    if (user_size > 0 && dump_size > user_size) {
+      dump_size = user_size;
+    }
   }
   
-  if (size > 0 && dump_size > size) {
-    dump_size = size;
-  }
-  
-  dump_pnt = (char *)user_pnt + offset;
+  dump_pnt = (char *)start_user + offset;
   if (IS_IN_HEAP(dump_pnt)) {
     out_len = expand_chars(dump_pnt, dump_size, out, sizeof(out));
-    dmalloc_message("Dump of '%#lx'%+d: '%.*s'",
-		    (unsigned long)user_pnt, offset, out_len, out);
+    dmalloc_message("  dump of '%#lx'%+d: '%.*s'",
+		    (unsigned long)start_user, offset, out_len, out);
   }
   else {
-    dmalloc_message("Dump of '%#lx'%+d failed: not in heap",
-		    (unsigned long)user_pnt, offset);
+    dmalloc_message("  dump of '%#lx'%+d failed: not in heap",
+		    (unsigned long)start_user, offset);
+  }
+  
+  /* find the previous pointer in case it ran over */
+  if (dmalloc_errno == ERROR_UNDER_FENCE && start_user != NULL) {
+    other_p = find_address((char *)start_user - FENCE_BOTTOM_SIZE - 1,
+			   1 /* not exact pointer */, skip_update);
+    if (other_p != NULL) {
+      dmalloc_message("  prev pointer '%#lx' (size %u) may have run over from '%s'",
+		      (unsigned long)other_p->sa_mem, other_p->sa_user_size,
+		      _dmalloc_chunk_desc_pnt(where_buf, sizeof(where_buf),
+					      other_p->sa_file,
+					      other_p->sa_line));
+    }
+  }
+  /* find the next pointer in case it ran under */
+  else if (dmalloc_errno == ERROR_OVER_FENCE
+	   && start_user != NULL
+	   && slot_p != NULL) {
+    other_p = find_address((char *)slot_p->sa_mem + slot_p->sa_total_size,
+			   1 /* not exact pointer */, skip_update);
+    if (other_p != NULL) {
+      dmalloc_message("  next pointer '%#lx' (size %u) may have run under from '%s'",
+		      (unsigned long)other_p->sa_mem, other_p->sa_user_size,
+		      _dmalloc_chunk_desc_pnt(where_buf, sizeof(where_buf),
+					      other_p->sa_file,
+					      other_p->sa_line));
+    }
   }
 }
 
@@ -1253,16 +1216,32 @@ static	void	clear_alloc(skip_alloc_t *slot_p, pnt_info_t *info_p,
   char	*start_p;
   int	num;
   
+  /*
+   * NOTE: The alloc blank flag is set so we blank a slot when it is
+   * allocated.  It used to be that the allocated spaces were blanked
+   * and the free spaces of the allocated chunk were blanked only if
+   * the FREE_BLANK flag was enabled.  Wrong!
+   */
+  
+  /*
+   * Set our slot blank flag if the flags are set now.  This will
+   * carry over with a realloc.
+   */
+  if (BIT_IS_SET(_dmalloc_flags, DEBUG_ALLOC_BLANK)
+      || BIT_IS_SET(_dmalloc_flags, DEBUG_CHECK_BLANK)) {
+    BIT_SET(slot_p->sa_flags, ALLOC_FLAG_BLANK);
+  }
+  
   /*
    * If we have a fence post protected valloc then there is almost a
    * full block at the front what is "free".  Set it with blank chars.
    */
-  num = (char *)info_p->pi_fence_bottom - (char *)info_p->pi_alloc_start;
-  if (num > 0 && (BIT_IS_SET(_dmalloc_flags, DEBUG_FREE_BLANK)
-		  || BIT_IS_SET(_dmalloc_flags, DEBUG_CHECK_BLANK))) {
-    memset(info_p->pi_alloc_start, FREE_BLANK_CHAR, num);
-    /* set our slot blank flag */
-    BIT_SET(slot_p->sa_flags, ALLOC_FLAG_BLANK);
+  if (info_p->pi_fence_b) {
+    num = (char *)info_p->pi_fence_bottom - (char *)info_p->pi_alloc_start;
+    /* alloc-blank NOT free-blank */
+    if (num > 0 && BIT_IS_SET(slot_p->sa_flags, ALLOC_FLAG_BLANK)) {
+      memset(info_p->pi_alloc_start, ALLOC_BLANK_CHAR, num);
+    }
   }
   
   /*
@@ -1276,8 +1255,7 @@ static	void	clear_alloc(skip_alloc_t *slot_p, pnt_info_t *info_p,
     if (func_id == DMALLOC_FUNC_CALLOC || func_id == DMALLOC_FUNC_RECALLOC) {
       memset(start_p, 0, num);
     }
-    else if (BIT_IS_SET(_dmalloc_flags, DEBUG_ALLOC_BLANK)
-	     || BIT_IS_SET(_dmalloc_flags, DEBUG_CHECK_BLANK)) {
+    else if (BIT_IS_SET(slot_p->sa_flags, ALLOC_FLAG_BLANK)) {
       memset(start_p, ALLOC_BLANK_CHAR, num);
     }
   }
@@ -1289,11 +1267,12 @@ static	void	clear_alloc(skip_alloc_t *slot_p, pnt_info_t *info_p,
   }
   
   /*
-   * now clear the rest of the block above any fence post space with
-   * free characters
+   * Now clear the rest of the block above any fence post space with
+   * free characters.
+   *
+   * NOTE alloc-blank NOT free-blank
    */
-  if (BIT_IS_SET(_dmalloc_flags, DEBUG_FREE_BLANK)
-      || BIT_IS_SET(_dmalloc_flags, DEBUG_CHECK_BLANK)) {
+  if (BIT_IS_SET(slot_p->sa_flags, ALLOC_FLAG_BLANK)) {
     
     if (info_p->pi_fence_b) {
       start_p = (char *)info_p->pi_fence_top + FENCE_TOP_SIZE;
@@ -1304,11 +1283,8 @@ static	void	clear_alloc(skip_alloc_t *slot_p, pnt_info_t *info_p,
     
     num = (char *)info_p->pi_alloc_bounds - start_p;
     if (num > 0) {
-      memset(start_p, FREE_BLANK_CHAR, num);
+      memset(start_p, ALLOC_BLANK_CHAR, num);
     }
-    
-    /* set our slot blank flag */
-    BIT_SET(slot_p->sa_flags, ALLOC_FLAG_BLANK);
   }
 }
 
@@ -1336,8 +1312,9 @@ static	int	create_divided_chunks(const unsigned int div_size)
   void		*mem, *bounds_p;
   
   /* allocate a 1 block chunk that we will cut up into pieces */
-  mem = allocate_memory(BLOCK_SIZE);
+  mem = _dmalloc_heap_alloc(BLOCK_SIZE);
   if (mem == HEAP_ALLOC_ERROR) {
+    /* error code set in _dmalloc_heap_alloc */
     return 0;
   }
   user_block_c++;
@@ -1470,7 +1447,8 @@ static	skip_alloc_t	*use_free_memory(const unsigned int size,
 static	skip_alloc_t	*get_divided_memory(const unsigned int size)
 {
   skip_alloc_t	*slot_p;
-  unsigned int	need_size, *bits_p;
+  unsigned int	need_size;
+  int		*bits_p;
   
   for (bits_p = bit_sizes;; bits_p++) {
     if (*bits_p >= size) {
@@ -1575,8 +1553,9 @@ static	skip_alloc_t	*get_memory(const unsigned int size)
   }
   
   /* allocate the memory necessary for the new blocks */
-  mem = allocate_memory(need_size);
+  mem = _dmalloc_heap_alloc(need_size);
   if (mem == HEAP_ALLOC_ERROR) {
+    /* error code set in _dmalloc_heap_alloc */
     return NULL;
   }
   user_block_c += block_n;
@@ -1653,8 +1632,9 @@ static	int	check_used_slot(const skip_alloc_t *slot_p,
    * If we have a valloc allocation then the _user_ pnt should be
    * block aligned otherwise the chunk_pnt should be.
    */
-  if (BIT_IS_SET(slot_p->sa_flags, ALLOC_FLAG_VALLOC)) {
-    if (! ON_BLOCK(pnt_info.pi_user_start)) {
+  if (pnt_info.pi_valloc_b) {
+    
+    if ((long)pnt_info.pi_user_start % BLOCK_SIZE != 0) {
       dmalloc_errno = ERROR_NOT_ON_BLOCK;
       return 0;
     }
@@ -1664,28 +1644,29 @@ static	int	check_used_slot(const skip_alloc_t *slot_p,
     }
     
     /* now check the below space to make sure it is still clear */
-    num = (char *)pnt_info.pi_fence_bottom - (char *)pnt_info.pi_alloc_start;
-    if (num > 0 && BIT_IS_SET(slot_p->sa_flags, ALLOC_FLAG_BLANK)) {
-      for (mem_p = pnt_info.pi_alloc_start;
-	   mem_p < (char *)pnt_info.pi_fence_bottom;
-	   mem_p++) {
-	if (*mem_p != FREE_BLANK_CHAR) {
-	  dmalloc_errno = ERROR_FREE_OVERWRITTEN;
-	  return 0;
+    if (pnt_info.pi_fence_b && pnt_info.pi_blanked_b) {
+      num = (char *)pnt_info.pi_fence_bottom - (char *)pnt_info.pi_alloc_start;
+      if (num > 0) {
+	for (mem_p = pnt_info.pi_alloc_start;
+	     mem_p < (char *)pnt_info.pi_fence_bottom;
+	     mem_p++) {
+	  if (*mem_p != ALLOC_BLANK_CHAR) {
+	    dmalloc_errno = ERROR_FREE_OVERWRITTEN;
+	    return 0;
+	  }
 	}
       }
     }
   }
   
   /* check out the fence-posts */
-  if (BIT_IS_SET(slot_p->sa_flags, ALLOC_FLAG_FENCE)
-      && (! fence_read(&pnt_info))) {
+  if (pnt_info.pi_fence_b && (! fence_read(&pnt_info))) {
     /* errno set in fence_read */
     return 0;
   }
   
   /* check above the allocation to see if it's been overwritten */
-  if (BIT_IS_SET(slot_p->sa_flags, ALLOC_FLAG_BLANK)) {
+  if (pnt_info.pi_blanked_b) {
     
     if (pnt_info.pi_fence_b) {
       mem_p = (char *)pnt_info.pi_fence_top + FENCE_TOP_SIZE;
@@ -1695,7 +1676,7 @@ static	int	check_used_slot(const skip_alloc_t *slot_p,
     }
     
     for (; mem_p < (char *)pnt_info.pi_alloc_bounds; mem_p++) {
-      if (*mem_p != FREE_BLANK_CHAR) {
+      if (*mem_p != ALLOC_BLANK_CHAR) {
 	dmalloc_errno = ERROR_FREE_OVERWRITTEN;
 	return 0;
       }
@@ -1969,16 +1950,14 @@ int	_dmalloc_chunk_read_info(const void *user_pnt, const char *where,
   slot_p = find_address(user_pnt, 0 /* not exact pointer */, skip_update);
   if (slot_p == NULL) {
     dmalloc_errno = ERROR_NOT_FOUND;
-    log_error_info(NULL, 0, NULL, 0, user_pnt, 0, NULL, where);
-    dmalloc_error("_dmalloc_chunk_read_info");
+    log_error_info(NULL, 0, user_pnt, NULL, "finding address in heap", where);
     return 0;
   }
   
   /* might as well check the pointer now */
   if (! check_used_slot(slot_p, user_pnt, 1 /* exact */)) {
     /* errno set in check_slot */
-    log_error_info(NULL, 0, NULL, 0, user_pnt, 0, NULL, where);
-    dmalloc_error("_dmalloc_chunk_read_info");
+    log_error_info(NULL, 0, user_pnt, slot_p, "checking pointer admin", where);
     return 0;
   }
   
@@ -2035,6 +2014,7 @@ int	_dmalloc_chunk_heap_check(void)
   skip_alloc_t	*slot_p;
   entry_block_t	*block_p;
   int		ret, level_c, checking_list_c = 0;
+  int		final = 1;
   
   if (BIT_IS_SET(_dmalloc_flags, DEBUG_LOG_TRANS)) {
     dmalloc_message("checking heap");
@@ -2176,18 +2156,27 @@ int	_dmalloc_chunk_heap_check(void)
     if (checking_list_c == 0) {
       ret = check_used_slot(slot_p, NULL /* no user pnt */,
 			    0 /* loose pnt checking */);
+      if (! ret) {
+	/* error set in check_slot */
+	log_error_info(NULL, 0, NULL, slot_p, "checking user pointer",
+		       "_dmalloc_chunk_heap_check");
+	/* not a critical error */
+	final = 0;
+      }
     }
     else {
       ret = check_free_slot(slot_p);
-    }
-    if (! ret) {
-      /* error set in check_slot */
-      dmalloc_error("_dmalloc_chunk_heap_check");
-      return 0;
+      if (! ret) {
+	/* error set in check_slot */
+	log_error_info(NULL, 0, NULL, slot_p, "checking free pointer",
+		       "_dmalloc_chunk_heap_check");
+	/* not a critical error */
+	final = 0;
+      }
     }
   }
   
-  return 1;
+  return final;
 }
 
 /*
@@ -2237,8 +2226,7 @@ int	_dmalloc_chunk_pnt_check(const char *func, const void *user_pnt,
   if (slot_p == NULL) {
     if (exact_b) {
       dmalloc_errno = ERROR_NOT_FOUND;
-      log_error_info(NULL, 0, NULL, 0, user_pnt, 0, NULL, "pointer-check");
-      dmalloc_error(func);
+      log_error_info(NULL, 0, user_pnt, NULL, "pointer-check", func);
       return 0;
     }
     else {
@@ -2250,9 +2238,7 @@ int	_dmalloc_chunk_pnt_check(const char *func, const void *user_pnt,
   /* now make sure that the user slot is valid */
   if (! check_used_slot(slot_p, user_pnt, exact_b)) {
     /* dmalloc_error set in check_used_slot */
-    log_error_info(NULL, 0, slot_p->sa_file, slot_p->sa_line,
-		   user_pnt, 0, NULL, "pointer-check");
-    dmalloc_error(func);
+    log_error_info(NULL, 0, user_pnt, slot_p, "pointer-check", func);
     return 0;
   }
   
@@ -2273,9 +2259,7 @@ int	_dmalloc_chunk_pnt_check(const char *func, const void *user_pnt,
     if ((char *)user_pnt < (char *)pnt_info.pi_user_start
 	|| (char *)user_pnt + min > (char *)pnt_info.pi_user_bounds) {
       dmalloc_errno = ERROR_WOULD_OVERWRITE;
-      log_error_info(NULL, 0, slot_p->sa_file, slot_p->sa_line,
-		     user_pnt, 0, NULL, "pointer-check");
-      dmalloc_error(func);
+      log_error_info(NULL, 0, user_pnt, slot_p, "pointer-check", func);
       return 0;
     }
   }
@@ -2345,9 +2329,8 @@ void	*_dmalloc_chunk_malloc(const char *file, const unsigned int line,
 #if ALLOW_ALLOC_ZERO_SIZE == 0
   if (size == 0) {
     dmalloc_errno = ERROR_BAD_SIZE;
-    log_error_info(file, line, NULL, 0, NULL, 0,
-		   "bad zero byte allocation request", "malloc");
-    dmalloc_error("_dmalloc_chunk_malloc");
+    log_error_info(file, line, NULL, NULL, "bad zero byte allocation request",
+		   "malloc");
     return MALLOC_ERROR;
   }
 #endif
@@ -2356,8 +2339,7 @@ void	*_dmalloc_chunk_malloc(const char *file, const unsigned int line,
   /* have we exceeded the upper bounds */
   if (size > LARGEST_ALLOCATION) {
     dmalloc_errno = ERROR_TOO_BIG;
-    log_error_info(file, line, NULL, 0, NULL, 0, NULL, "malloc");
-    dmalloc_error("_dmalloc_chunk_malloc");
+    log_error_info(file, line, NULL, NULL, "allocation too big", "malloc");
     return MALLOC_ERROR;
   }
 #endif
@@ -2407,7 +2389,7 @@ void	*_dmalloc_chunk_malloc(const char *file, const unsigned int line,
   
   get_pnt_info(slot_p, &pnt_info);
   
-  /* not clear the allocation */
+  /* clear the allocation */
   clear_alloc(slot_p, &pnt_info, 0 /* no old-size */, func_id);
   
   slot_p->sa_file = file;
@@ -2535,9 +2517,7 @@ int	_dmalloc_chunk_free(const char *file, const unsigned int line,
      */
     if (BIT_IS_SET(_dmalloc_flags, DEBUG_ERROR_FREE_NULL)) {
       dmalloc_errno = ERROR_IS_NULL;
-      log_error_info(file, line, NULL, 0, user_pnt, 0, "invalid pointer",
-		     "free");
-      dmalloc_error("_dmalloc_chunk_free");
+      log_error_info(file, line, user_pnt, NULL, "invalid 0L pointer", "free");
       return FREE_ERROR;
     }
     
@@ -2554,16 +2534,15 @@ int	_dmalloc_chunk_free(const char *file, const unsigned int line,
   if (slot_p == NULL) {
     /* not found */
     dmalloc_errno = ERROR_NOT_FOUND;
-    log_error_info(file, line, NULL, 0, user_pnt, 0, NULL, "free");
-    dmalloc_error("_dmalloc_chunk_free");
+    log_error_info(file, line, user_pnt, NULL, "finding address in heap",
+		   "free");
     return FREE_ERROR;
   }
   
   if (! check_used_slot(slot_p, user_pnt, 1 /* exact pnt */)) {
     /* error set in check slot */
-    log_error_info(file, line, slot_p->sa_file, slot_p->sa_line, user_pnt,
-		   0, NULL, "free");
-    dmalloc_error("_dmalloc_chunk_free");
+    log_error_info(file, line, user_pnt, slot_p, "checking pointer admin",
+		   "free");
     return FREE_ERROR;
   }
   
@@ -2695,9 +2674,8 @@ void	*_dmalloc_chunk_realloc(const char *file, const unsigned int line,
 #if ALLOW_ALLOC_ZERO_SIZE == 0
   if (new_size == 0) {
     dmalloc_errno = ERROR_BAD_SIZE;
-    log_error_info(file, line, NULL, 0, NULL, 0,
-		   "bad zero byte allocation request", "realloc");
-    dmalloc_error("_dmalloc_chunk_realloc");
+    log_error_info(file, line, NULL, NULL, "bad zero byte allocation request",
+		   "realloc");
     return REALLOC_ERROR;
   }
 #endif
@@ -2705,9 +2683,8 @@ void	*_dmalloc_chunk_realloc(const char *file, const unsigned int line,
   /* by now malloc.c should have taken care of the realloc(NULL) case */
   if (old_user_pnt == NULL) {
     dmalloc_errno = ERROR_IS_NULL;
-    log_error_info(file, line, NULL, 0, old_user_pnt, 0, "invalid pointer",
+    log_error_info(file, line, old_user_pnt, NULL, "invalid pointer",
 		   "realloc");
-    dmalloc_error("_dmalloc_chunk_realloc");
     return REALLOC_ERROR;
   }
   
@@ -2715,9 +2692,8 @@ void	*_dmalloc_chunk_realloc(const char *file, const unsigned int line,
   slot_p = find_address(old_user_pnt, 0 /* not exact pointer */, skip_update);
   if (slot_p == NULL) {
     dmalloc_errno = ERROR_NOT_FOUND;
-    log_error_info(file, line, NULL, 0, old_user_pnt, 0, NULL,
-		   "_dmalloc_chunk_realloc");
-    dmalloc_error("_dmalloc_chunk_realloc");
+    log_error_info(file, line, old_user_pnt, NULL, "finding address in heap",
+		   "realloc");
     return 0;
   }
   
@@ -2846,35 +2822,32 @@ void	*_dmalloc_chunk_realloc(const char *file, const unsigned int line,
  */
 void	_dmalloc_chunk_log_stats(void)
 {
-  unsigned long	overhead, tot_space, ext_space;
+  unsigned long	overhead, user_space, tot_space;
   
   dmalloc_message("Dumping Chunk Statistics:");
   
-  tot_space = alloc_current + free_space_bytes;
+  tot_space = (user_block_c + admin_block_c) * BLOCK_SIZE;
+  user_space = alloc_current + free_space_bytes;
   overhead = admin_block_c * BLOCK_SIZE;
   
   /* version information */
-  dmalloc_message("basic-block %d bytes, alignment %d bytes, heap grows %s",
-		  BLOCK_SIZE, ALLOCATION_ALIGNMENT,
-		  (HEAP_GROWS_UP ? "up" : "down"));
+  dmalloc_message("basic-block %d bytes, alignment %d bytes",
+		  BLOCK_SIZE, ALLOCATION_ALIGNMENT);
   
   /* general heap information with blocks */
   dmalloc_message("heap address range: %#lx to %#lx, %ld bytes",
-		  (unsigned long)_dmalloc_heap_base,
-		  (unsigned long)_dmalloc_heap_last, (long)HEAP_SIZE);
+		  (unsigned long)_dmalloc_heap_low,
+		  (unsigned long)_dmalloc_heap_high,
+		  (unsigned long)_dmalloc_heap_high -
+		  (unsigned long)_dmalloc_heap_low);
   dmalloc_message("    user blocks: %ld blocks, %ld bytes (%ld%%)",
-		  user_block_c, tot_space,
-		  (HEAP_SIZE == 0 ? 0 : tot_space / (HEAP_SIZE / 100)));
+		  user_block_c, user_space,
+		  (tot_space < 100 ? 0 : user_space / (tot_space / 100)));
   dmalloc_message("   admin blocks: %ld blocks, %ld bytes (%ld%%)",
 		  admin_block_c, overhead,
-		  (HEAP_SIZE == 0 ? 0 : overhead / (HEAP_SIZE / 100)));
-  ext_space = extern_block_c * BLOCK_SIZE;
-  dmalloc_message("external blocks: %ld blocks, %ld bytes (%ld%%)",
-		  extern_block_c, ext_space,
-		  (HEAP_SIZE == 0 ? 0 : ext_space / (HEAP_SIZE / 100)));
-  tot_space = (user_block_c + admin_block_c + extern_block_c) * BLOCK_SIZE;
+		  (tot_space < 100 ? 0 : overhead / (tot_space / 100)));
   dmalloc_message("   total blocks: %ld blocks, %ld bytes",
-		  user_block_c + admin_block_c + extern_block_c, tot_space);
+		  user_block_c + admin_block_c, tot_space);
   
   dmalloc_message("heap checked %ld", heap_check_c);
   
