@@ -35,11 +35,6 @@
 #include "dmalloc_argv_loc.h"
 #include "compat.h"
 
-#if INCLUDE_RCS_IDS
-static	char	*rcs_id =
-  "$Id: dmalloc_argv.c,v 1.10 2000/05/15 15:42:58 gray Exp $";
-#endif
-
 /* internal routines */
 static	void	do_list(argv_t *grid, const int arg_c, char **argv,
 			argv_t **queue_list, int *queue_head_p,
@@ -496,7 +491,7 @@ static	int	expand_buf(const void *buf, const int buf_size,
       if (out_p + 2 >= max_p) {
 	break;
       }
-      (void)sprintf(out_p, "\\%c", *(spec_p - 1));
+      (void)loc_snprintf(out_p, max_p - out_p, "\\%c", *(spec_p - 1));
       out_p += 2;
       continue;
     }
@@ -513,7 +508,7 @@ static	int	expand_buf(const void *buf, const int buf_size,
       if (out_p + 4 >= max_p) {
 	break;
       }
-      (void)sprintf(out_p, "\\%03o", *buf_p);
+      (void)loc_snprintf(out_p, max_p - out_p, "\\%03o", *buf_p);
       out_p += 4;
     }
   }
@@ -1536,42 +1531,42 @@ static	int	value_to_string(const ARGV_PNT var, const unsigned int type,
     break;
     
   case ARGV_SHORT:
-    (void)sprintf(buf, "%d", *(short *)var);
+    (void)loc_snprintf(buf, buf_size, "%d", *(short *)var);
     len = strlen(buf);
     break;
     
   case ARGV_U_SHORT:
-    (void)sprintf(buf, "%d", *(unsigned short *)var);
+    (void)loc_snprintf(buf, buf_size, "%d", *(unsigned short *)var);
     len = strlen(buf);
     break;
     
   case ARGV_INT:
-    (void)sprintf(buf, "%d", *(int *)var);
+    (void)loc_snprintf(buf, buf_size, "%d", *(int *)var);
     len = strlen(buf);
     break;
     
   case ARGV_U_INT:
-    (void)sprintf(buf, "%u", *(unsigned int *)var);
+    (void)loc_snprintf(buf, buf_size, "%u", *(unsigned int *)var);
     len = strlen(buf);
     break;
     
   case ARGV_LONG:
-    (void)sprintf(buf, "%ld", *(long *)var);
+    (void)loc_snprintf(buf, buf_size, "%ld", *(long *)var);
     len = strlen(buf);
     break;
     
   case ARGV_U_LONG:
-    (void)sprintf(buf, "%lu", *(unsigned long *)var);
+    (void)loc_snprintf(buf, buf_size, "%lu", *(unsigned long *)var);
     len = strlen(buf);
     break;
     
   case ARGV_FLOAT:
-    (void)sprintf(buf, "%f", *(float *)var);
+    (void)loc_snprintf(buf, buf_size, "%f", *(float *)var);
     len = strlen(buf);
     break;
     
   case ARGV_DOUBLE:
-    (void)sprintf(buf, "%f", *(double *)var);
+    (void)loc_snprintf(buf, buf_size, "%f", *(double *)var);
     len = strlen(buf);
     break;
     
@@ -1579,12 +1574,14 @@ static	int	value_to_string(const ARGV_PNT var, const unsigned int type,
   case ARGV_BIN:
     {
       int	bit_c, bit, first_b = ARGV_FALSE;
-      char	binary[2 + 128 + 1], *bin_p = binary;
+      char	binary[2 + 128 + 1], *bounds, *bin_p;
       
       if (*(int *)var == 0) {
 	strncpy(buf, "0", buf_size);
       }
       else {
+	bin_p = binary;
+	bounds = binary + sizeof(binary);
 	
 	/* initially write binary number into tmp buffer, then copy into out */
 	*bin_p++ = '0';
@@ -1605,7 +1602,7 @@ static	int	value_to_string(const ARGV_PNT var, const unsigned int type,
 	}
 	
 	/* add on the decimal equivalent */ 
-	(void)sprintf(bin_p, " (%d)", *(int *)var);
+	(void)loc_snprintf(bin_p, bounds - bin_p, " (%d)", *(int *)var);
 	/* find the \0 at end */ 
 	for (; *bin_p != '\0'; bin_p++) {
 	}
@@ -1625,7 +1622,7 @@ static	int	value_to_string(const ARGV_PNT var, const unsigned int type,
       buf[buf_size - 1] = '\0';
     }
     else {
-      (void)sprintf(buf, "%#o (%d)", *(int *)var, *(int *)var);
+      (void)loc_snprintf(buf, buf_size, "%#o (%d)", *(int *)var, *(int *)var);
     }
     len = strlen(buf);
     break;
@@ -1635,13 +1632,13 @@ static	int	value_to_string(const ARGV_PNT var, const unsigned int type,
       (void)strcpy(buf, "0");
     }
     else {
-      (void)sprintf(buf, "%#x (%d)", *(int *)var, *(int *)var);
+      (void)loc_snprintf(buf, buf_size, "%#x (%d)", *(int *)var, *(int *)var);
     }
     len = strlen(buf);
     break;
     
   case ARGV_INCR:
-    (void)sprintf(buf, "%d", *(int *)var);
+    (void)loc_snprintf(buf, buf_size, "%d", *(int *)var);
     len = strlen(buf);
     break;
     
@@ -1654,18 +1651,18 @@ static	int	value_to_string(const ARGV_PNT var, const unsigned int type,
       }
       else if (val % (1024 * 1024 * 1024) == 0) {
 	morf = val / (1024 * 1024 * 1024);
-	(void)sprintf(buf, "%ldg (%ld)", morf, val);
+	(void)loc_snprintf(buf, buf_size, "%ldg (%ld)", morf, val);
       }
       else if (val % (1024 * 1024) == 0) {
 	morf = val / (1024 * 1024);
-	(void)sprintf(buf, "%ldm (%ld)", morf, val);
+	(void)loc_snprintf(buf, buf_size, "%ldm (%ld)", morf, val);
       }
       else if (val % 1024 == 0) {
 	morf = val / 1024;
-	(void)sprintf(buf, "%ldk (%ld)", morf, val);
+	(void)loc_snprintf(buf, buf_size, "%ldk (%ld)", morf, val);
       }
       else {
-	(void)sprintf(buf, "%ld", val);
+	(void)loc_snprintf(buf, buf_size, "%ld", val);
       }
       
       len = strlen(buf);
@@ -1681,18 +1678,18 @@ static	int	value_to_string(const ARGV_PNT var, const unsigned int type,
       }
       else if (val % (1024 * 1024 * 1024) == 0) {
 	morf = val / (1024 * 1024 * 1024);
-	(void)sprintf(buf, "%ldg (%ld)", morf, val);
+	(void)loc_snprintf(buf, buf_size, "%ldg (%ld)", morf, val);
       }
       else if (val % (1024 * 1024) == 0) {
 	morf = val / (1024 * 1024);
-	(void)sprintf(buf, "%ldm (%ld)", morf, val);
+	(void)loc_snprintf(buf, buf_size, "%ldm (%ld)", morf, val);
       }
       else if (val % 1024 == 0) {
 	morf = val / 1024;
-	(void)sprintf(buf, "%ldk (%ld)", morf, val);
+	(void)loc_snprintf(buf, buf_size, "%ldk (%ld)", morf, val);
       }
       else {
-	(void)sprintf(buf, "%ld", val);
+	(void)loc_snprintf(buf, buf_size, "%ld", val);
       }
       
       len = strlen(buf);
@@ -2360,8 +2357,7 @@ static	void	do_arg(argv_t *grid, argv_t *match_p, const char *close_p,
  * Examine an argument string to see if it really is a negative number
  * being passed into a previously specified argument.
  *
- * Thanks much to Nick Kisseberth <nkissebe@hera.itg.uiuc.edu> for
- * pointing out this oversight.
+ * Thanks much to Nick Kisseberth for pointing out this oversight.
  *
  * RETURNS:
  *
@@ -2695,8 +2691,7 @@ static	void	do_list(argv_t *grid, const int arg_c, char **argv,
 	   * allow values with negative signs if we are at the start
 	   * of an argument list, and if the argument is a number, and
 	   * we already have a variable looking for a value.  Thanks
-	   * to Nick Kisseberth <nkissebe@hera.itg.uiuc.edu> for
-	   * pointing out this oversight.
+	   * to Nick Kisseberth for pointing out this oversight.
 	   */
 	  if (char_c == 0 && is_number(*arg_p)
 	      && *queue_head_p > *queue_tail_p) {
@@ -2844,10 +2839,10 @@ static	int	do_env_args(argv_t *args, argv_t **queue_list,
 			    int *queue_head_p, int *queue_tail_p, int *okay_bp)
 {
   int	env_c, env_n;
-  char	**vect_p, env_name[256], *environ_p;
+  char	**vect_p, env_name[1024], *environ_p;
   
   /* create the env variable */
-  (void)sprintf(env_name, ENVIRON_FORMAT, argv_program);
+  (void)loc_snprintf(env_name, sizeof(env_name), ENVIRON_FORMAT, argv_program);
   
   /* NOTE: by default the env name is all uppercase */
   for (environ_p = env_name; *environ_p != '\0'; environ_p++) {
@@ -2903,7 +2898,7 @@ static	int	do_env_args(argv_t *args, argv_t **queue_list,
 static	int	process_env(void)
 {
   static int	done_b = ARGV_FALSE;
-  char		*environ, *tok_p, *env_p;
+  char		*env_val, *tok_p, *env_p;
   int		len;
   
   /* make sure we only do this once */
@@ -2914,18 +2909,18 @@ static	int	process_env(void)
   done_b = ARGV_TRUE;
   
   /* get the argv information */
-  environ = getenv(GLOBAL_NAME);
-  if (environ == NULL) {
+  env_val = getenv(GLOBAL_NAME);
+  if (env_val == NULL) {
     return NOERROR;
   }
   
   /* save a copy of it */
-  environ = string_copy(environ);
-  if (environ == NULL) {
+  env_val = string_copy(env_val);
+  if (env_val == NULL) {
     return ERROR;
   }
   
-  env_p = environ;
+  env_p = env_val;
   
   for (;;) {
     tok_p = strsep(&env_p, " \t,:");
@@ -3094,7 +3089,7 @@ static	int	process_env(void)
     }
   }
   
-  free(environ);
+  free(env_val);
   return NOERROR;
 }
 
@@ -3622,7 +3617,8 @@ int	argv_value_string(const argv_t *argv_entry_p, char *buf,
 	ret = len;
       }
       else {
-	(void)sprintf(details, " (1st of %d entries)", arr_p->aa_entry_n);
+	(void)loc_snprintf(details, sizeof(details), " (1st of %d entries)",
+			   arr_p->aa_entry_n);
 	strncpy(buf + len, details, buf_size - len);
 	buf[buf_size - 1] = '\0';
 	ret = strlen(buf);
