@@ -18,7 +18,7 @@
  *
  * The author may be contacted via http://dmalloc.com/
  *
- * $Id: heap.c,v 1.58 2000/11/13 15:46:51 gray Exp $
+ * $Id: heap.c,v 1.68 2004/10/12 19:38:57 gray Exp $
  */
 
 /*
@@ -29,6 +29,12 @@
 #if HAVE_UNISTD_H
 # include <unistd.h>				/* for write */
 #endif
+#if HAVE_SYS_TYPES_H
+#  include <sys/types.h>
+#endif
+#if HAVE_SYS_MMAN_H
+#  include <sys/mman.h>				/* for mmap stuff */
+#endif
 
 #define DMALLOC_DISABLE
 
@@ -37,42 +43,74 @@
 
 #include "chunk.h"
 #include "compat.h"
-#include "debug_val.h"
+#include "debug_tok.h"
 #include "error.h"
 #include "error_val.h"
 #include "heap.h"
 #include "dmalloc_loc.h"
 
-#if INCLUDE_RCS_IDS
-#if IDENT_WORKS
-#ident "$Id: heap.c,v 1.58 2000/11/13 15:46:51 gray Exp $"
-#else
-static	char	*rcs_id =
-  "$Id: heap.c,v 1.58 2000/11/13 15:46:51 gray Exp $";
-#endif
-#endif
-
 #define SBRK_ERROR	((char *)-1)		/* sbrk error code */
 
 /* exported variables */
-void	*_heap_base = NULL;			/* base of our heap */
-void	*_heap_last = NULL;			/* end of our heap */
+void		*_dmalloc_heap_low = NULL;	/* base of our heap */
+void		*_dmalloc_heap_high = NULL;	/* end of our heap */
 
 /****************************** local functions ******************************/
 
 /*
- * Increment the heap INCR bytes with sbrk
+ * static void *heap_extend
+ *
+ * DESCRIPTION:
+ *
+ * Get more bytes from the system functions.
+ *
+ * RETURNS:
+ *
+ * Success - Valid pointer.
+ *
+ * Failure - NULL
+ *
+ * ARGUMENTS:
+ *
+ * incr -> Number of bytes we need.
  */
 static	void	*heap_extend(const int incr)
 {
   void	*ret = SBRK_ERROR;
+  char	*high;
   
-  if (BIT_IS_SET(_dmalloc_flags, DEBUG_LOG_ADMIN)) {
-    _dmalloc_message("extending heap space by %d bytes", incr);
+#if INTERNAL_MEMORY_SPACE
+  {
+    static char	block_o_bytes[INTERNAL_MEMORY_SPACE];
+    static char *bounds_p = block_o_bytes + sizeof(block_o_bytes);
+    static char *block_p = block_o_bytes;
+    
+    if (block_p + incr >= bounds_p) {
+      ret = SBRK_ERROR;
+    }
+    else {
+      ret = block_p;
+      block_p += incr;
+    }
   }
-  
+#else
+#if HAVE_MMAP && USE_MMAP
+#if MAP_ANON
+  /* if we have and can use mmap, then do so */
+  ret = mmap(0L, incr, PROT_READ | PROT_WRITE | PROT_EXEC,
+	     MAP_PRIVATE | MAP_ANON, -1 /* no fd */, 0 /* no offset */);
+#else
+#endif
+  if (ret == MAP_FAILED) {
+    ret = SBRK_ERROR;
+  }
+#else
 #if HAVE_SBRK
   ret = sbrk(incr);
+#endif /* if HAVE_SBRK */
+#endif /* if not HAVE_MMAP && USE_MMAP */
+#endif /* if not INTERNAL_MEMORY_SPACE */
+  
   if (ret == SBRK_ERROR) {
     if (BIT_IS_SET(_dmalloc_flags, DEBUG_CATCH_NULL)) {
       char	str[128];
@@ -85,7 +123,20 @@ static	void	*heap_extend(const int incr)
     dmalloc_errno = ERROR_ALLOC_FAILED;
     dmalloc_error("heap_extend");
   }
-#endif
+  
+  if (_dmalloc_heap_low == NULL || (char *)ret < (char *)_dmalloc_heap_low) {
+    _dmalloc_heap_low = ret;
+  }
+  high = (char *)ret + incr;
+  if (high > (char *)_dmalloc_heap_high) {
+    _dmalloc_heap_high = high;
+  }
+  
+  if (BIT_IS_SET(_dmalloc_flags, DEBUG_LOG_ADMIN)) {
+    dmalloc_message("extended heap space by %d bytes [%#lx, %#lx]",
+		    incr, (unsigned long)_dmalloc_heap_low,
+		    (unsigned long)_dmalloc_heap_high);
+  }
   
   return ret;
 }
@@ -109,48 +160,37 @@ static	void	*heap_extend(const int incr)
  *
  * None.
  */
-int	_heap_startup(void)
+int	_dmalloc_heap_startup(void)
 {
-  long		diff;
-  
-  _heap_base = heap_extend(0);
-  if (_heap_base == SBRK_ERROR) {
-    return 0;
-  }
-  
-  /* align the heap-base */
-  diff = BLOCK_SIZE - ((long)_heap_base % BLOCK_SIZE);
-  if (diff == BLOCK_SIZE) {
-    diff = 0;
-  }
-  
-  if (diff > 0) {
-    if (heap_extend(diff) == SBRK_ERROR) {
-      return 0;
-    }
-    _heap_base = (char *)HEAP_INCR(_heap_base, diff);
-  }
-  
-  _heap_last = _heap_base;
-  
   return 1;
 }
 
 /*
- * Function to get SIZE memory bytes from the end of the heap.  it
- * returns a pointer to any external blocks in EXTERN_P and the number
- * of blocks in EXTERN_CP.
+ * void *_dmalloc_heap_alloc
+ *
+ * DESCRIPTION:
+ *
+ * Function to get memory bytes from the heap.
+ *
+ * RETURNS:
+ *
+ * Success - Valid pointer.
+ *
+ * Failure - NULL
+ *
+ * ARGUMENTS:
+ *
+ * size -> Number of bytes we need.
  */
-void	*_heap_alloc(const unsigned int size, void **extern_p,
-		     int *extern_cp)
+void	*_dmalloc_heap_alloc(const unsigned int size)
 {
-  void		*heap_new, *heap_diff;
-  long		diff;
-  int		block_n = 0;
+  void	*heap_new, *heap_diff;
+  long	diff;
   
-  /* set our external memory pointer to where the heap should be */ 
-  if (extern_p != NULL) {
-    *extern_p = _heap_last;
+  if (size == 0) {
+    dmalloc_errno = ERROR_BAD_SIZE;
+    dmalloc_error("_dmalloc_heap_alloc");
+    return HEAP_ALLOC_ERROR;
   }
   
   while (1) {
@@ -161,46 +201,13 @@ void	*_heap_alloc(const unsigned int size, void **extern_p,
       return HEAP_ALLOC_ERROR;
     }
     
-    /* is the heap linear? */
-    if (heap_new == _heap_last) {
-      _heap_last = HEAP_INCR(heap_new, size);
-      if (extern_cp != NULL) {
-	*extern_cp = 0;
-      }
-      return heap_new;
-    }
-    
-    /* if we went down then this is a real error! */
-    if ((! IS_GROWTH(heap_new, _heap_last))
-	|| BIT_IS_SET(_dmalloc_flags, DEBUG_FORCE_LINEAR)) {
-      dmalloc_errno = ERROR_ALLOC_NONLINEAR;
-      dmalloc_error("_heap_alloc");
-      return HEAP_ALLOC_ERROR;
-    }
-    
-    /* adjust chunk admin information to align to blocksize */
-    block_n += BLOCKS_BETWEEN(heap_new, _heap_last);
-    
-    /* move heap last forward */
-    _heap_last = HEAP_INCR(heap_new, size);
-    
     /* calculate bytes needed to align to block boundary */
-    diff = BLOCK_SIZE - ((long)heap_new % BLOCK_SIZE);
-    if (diff == BLOCK_SIZE) {
-      if (BIT_IS_SET(_dmalloc_flags, DEBUG_LOG_TRANS)) {
-	_dmalloc_message("corrected non-linear heap for %d blocks", block_n);
-      }
-      /* if external sbrk asked for full block(s) then no need to correct */
+    diff = (long)heap_new % BLOCK_SIZE;
+    if (diff == 0) {
+      /* if we are already aligned then we are all set */
       break;
     }
-    
-    /* account for the partial block that we need to fill */
-    block_n++;
-    
-    if (BIT_IS_SET(_dmalloc_flags, DEBUG_LOG_TRANS)) {
-      _dmalloc_message("corrected non-linear non-aligned heap for %d blocks",
-		       block_n);
-    }
+    diff = BLOCK_SIZE - diff;
     
     /* shift the heap a bit to account for non block alignment */
     heap_diff = heap_extend(diff);
@@ -208,30 +215,25 @@ void	*_heap_alloc(const unsigned int size, void **extern_p,
       return HEAP_ALLOC_ERROR;
     }
     
-    /* if we got what we expected, then we are done */
-    if (heap_diff == _heap_last) {
-      /* shift the new pointer up to align it */
-      heap_new = HEAP_INCR(heap_new, diff);
-      /* move the heap last pointer past the diff section */
-      _heap_last = HEAP_INCR(heap_diff, diff);
+    /* if heap-diff went down then our stack grows down */
+    if ((char *)heap_diff + diff == (char *)heap_new) {
+      heap_new = heap_diff;
+      break;
+    }
+    else if ((char *)heap_new + size == (char *)heap_diff) {
+      /* shift up our heap to align with the block */
+      heap_new = (char *)heap_new + diff;
       break;
     }
     
     /*
      * We may have a wierd sbrk race condition here.  We hope that we
      * are not just majorly confused which may mean that we sbrk till
-     * the cows come home -- or we die from lack of memory.
+     * the cows come home and die from lack of memory.
      */
     
-    /* move the heap last pointer past the diff section */
-    _heap_last = HEAP_INCR(heap_diff, diff);
-    
     /* start over again */
   }
   
-  if (extern_cp != NULL) {
-    *extern_cp = block_n;
-  }
-  
   return heap_new;
 }
