@@ -18,7 +18,7 @@
  *
  * The author may be contacted via http://dmalloc.com/
  *
- * $Id: malloc.c,v 1.178 2004/01/28 14:30:40 gray Exp $
+ * $Id: malloc.c,v 1.183 2004/09/13 22:37:36 gray Exp $
  */
 
 /*
@@ -77,7 +77,7 @@
 #endif
 #endif
 
-#if SIGNAL_OKAY
+#if SIGNAL_OKAY && HAVE_SIGNAL_H
 #include <signal.h>
 #endif
 
@@ -100,7 +100,7 @@
 #if IDENT_WORKS
 #ident "@(#) $Information: lock-threads is enabled $"
 #else
-static char *information = "@(#) $Information: lock-threads is enabled $"
+static char *information = "@(#) $Information: lock-threads is enabled $";
 #endif
 #endif
 
@@ -1013,6 +1013,14 @@ char	*dmalloc_strdup(const char *file, const int line,
   DMALLOC_SIZE	len;
   char		*new_string;
   
+  /* check the arguments */
+  if (BIT_IS_SET(_dmalloc_flags, DEBUG_CHECK_FUNCS)) {
+    if (! dmalloc_verify_pnt(file, line, "strdup", string,
+			     0 /* not exact */, -1)) {
+      dmalloc_message("bad pointer argument found in strdup");
+    }
+  }
+  
   len = strlen(string) + 1;
   
   new_string = dmalloc_malloc(file, line, len, DMALLOC_FUNC_STRDUP,
@@ -1238,6 +1246,14 @@ char	*strdup(const char *string)
   
   GET_RET_ADDR(file);
   
+  /* check the arguments */
+  if (BIT_IS_SET(_dmalloc_flags, DEBUG_CHECK_FUNCS)) {
+    if (! dmalloc_verify_pnt(file, 0 /* no line */, "strdup", string,
+			     0 /* not exact */, -1)) {
+      dmalloc_message("bad pointer argument found in strdup");
+    }
+  }
+  
   /* len + \0 */
   len = strlen(string) + 1;
   
@@ -1276,7 +1292,7 @@ DMALLOC_FREE_RET	free(DMALLOC_PNT pnt)
   GET_RET_ADDR(file);
   ret = dmalloc_free(file, DMALLOC_DEFAULT_LINE, pnt, DMALLOC_FUNC_FREE);
   
-#if (defined(__STDC__) && __STDC__ == 1) || defined(__cplusplus)
+#if (defined(__STDC__) && __STDC__ == 1) || defined(__cplusplus) || defined(STDC_HEADERS)
 #else
   return ret;
 #endif
@@ -1307,7 +1323,7 @@ DMALLOC_FREE_RET	cfree(DMALLOC_PNT pnt)
   GET_RET_ADDR(file);
   ret = dmalloc_free(file, DMALLOC_DEFAULT_LINE, pnt, DMALLOC_FUNC_CFREE);
   
-#if (defined(__STDC__) && __STDC__ == 1) || defined(__cplusplus)
+#if (defined(__STDC__) && __STDC__ == 1) || defined(__cplusplus) || defined(STDC_HEADERS)
 #else
   return ret;
 #endif
@@ -1321,7 +1337,7 @@ DMALLOC_FREE_RET	cfree(DMALLOC_PNT pnt)
  * DESCRIPTION:
  *
  * Verify a pointer which has previously been allocated by the
- * library.
+ * library or check the entire heap.
  *
  * RETURNS:
  *
